#1.1 loading the data
library(WGCNA)
library("flashClust")
options(stringsAsFactors = FALSE);
enableWGCNAThreads()

#1.2  Checking data for excessive missing values and identification of outlier samples
gsg = goodSamplesGenes(c4, verbose = 3); #c4 is expression matrix
gsg$allOK
sampleTree = flashClust(dist(c4), method = "average"); #Check Outliers

#1.3 loading Phenodata (only stage data)
datTraits <- read.table("phenodata.txt", sep='\t', row.names=1, header=TRUE)

#1.4 Co-expression similarity and adjacency
softPower = 14
ADJ = adjacency(c4,power=softPower, type="signed")
diag(ADJ) = 0	
dissTOM    = 1-TOMsimilarity(ADJ, TOMType="signed")	
geneTree   = flashClust(as.dist(dissTOM),method="average")

#1.5 Cut the tree to choose a larger number of small modules
minModuleSize = 15
dynamicMods = cutreeDynamic(dendro = geneTree, distM = dissTOM,deepSplit = 4, pamRespectsDendro = FALSE, minClusterSize = minModuleSize);
table(dynamicMods)
dynamicColors = labels2colors(dynamicMods)
table(dynamicColors)

#1.6 Merging of modules whose expression profiles are very similar
# Calculate eigengenes
MEList = moduleEigengenes(c4, colors = dynamicColors)
MEs = MEList$eigengenes
# Calculate dissimilarity of module eigengenes
MEDiss = 1-cor(MEs);
# Cluster module eigengenes
METree = flashClust(as.dist(MEDiss), method = "average");
# Call an automatic merging function
merge = mergeCloseModules(c4, dynamicColors, cutHeight = MEDissThres, verbose = 3)
# The merged module colors
mergedColors <-merge$colors;
# Eigengenes of the new merged modules:
mergedMEs <- merge$newMEs;

moduleColors <- mergedColors # Rename to moduleColors
# Construct numerical labels corresponding to the colors
colorOrder = c("grey", standardColors(50));
moduleLabels = match(moduleColors, colorOrder)-1;
MEs = mergedMEs;
table(moduleColors)

#1.7 Relating modules to sample classification
nGenes = ncol(c4);
nSamples = nrow(c4);
# Recalculate MEs with color labels
MEs0 = moduleEigengenes(c4, moduleColors)$eigengenes
MEs = orderMEs(MEs0)
moduleTraitCor = cor(MEs, datTraits, use = "p");
moduleTraitCor
moduleTraitPvalue = corPvalueStudent(moduleTraitCor, nSamples);
moduleTraitPvalue
cbind(moduleTraitCor, moduleTraitPvalue)->module_trait_relationship
textMatrix =paste(signif(moduleTraitCor, 2),"\n(",signif(moduleTraitPvalue, 1),")", sep ="");
dim(textMatrix) =dim(moduleTraitCor)
png(file = "Module_trait_relationship.png", width = 5, height = 5, units = 'in', res = 300);
labeledHeatmap(Matrix = moduleTraitCor,xLabels = names(datTraits),yLabels = substring(names(MEs), 3),ySymbols = names(MEs),colorLabels = FALSE,colors = blueWhiteRed(50),textMatrix = textMatrix,setStdMargins = TRUE,cex.text = 1,zlim = c(-1,1),main = paste("Module-trait relationships"))
dev.off()
as.data.frame(cbind(moduleTraitCor, moduleTraitPvalue))->correlation_unadj_p_val
colnames(correlation_unadj_p_val)<-c("correlation", "pvalue")
correlation_unadj_p_val$pvalue->p
p.adjust(p, method="fdr", n=length(p))->p_adj_p
format(p_adj_p,scientific=TRUE)->p_adj_p_exponent
as.data.frame(cbind(correlation_unadj_p_val, p_adj_p_exponent))->module_trait_relationship
colnames(module_trait_relationship)<-c("correlation", "pvalue", "adjusted_pvalue")
write.table(module_trait_relationship, file="module_trait_relationship.txt", sep="\t", quote=F)

#Final modules selected: BLACK (+ve), GREEN (-ve), and RED (-ve)

#1.8. Which are the genes in these network modules
TOMMY = TOMsimilarityFromExpr(c4, power = 14, networkType = "signed", TOMType = "signed"); #power = 12 as used from earlier step
table(moduleColors) #5
II<-colnames(c4)
MOD_b =  "black"
MOD_g =  "green"
MOD_r =  "red"
#black module
inModule_b = is.finite(match(moduleColors, MOD_b))
modProbes_b = II[inModule_b]; 
modProbes_b
modTOM_b = TOMMY[inModule_b, inModule_b];
dimnames(modTOM_b) = list(modProbes_b, modProbes_b) 
dimnames(modTOM_b)
length(rownames(modTOM_b))
rownames(modTOM_b)->black_genes
#green module
inModule_g = is.finite(match(moduleColors, MOD_g))
modProbes_g = II[inModule_g]; 
modProbes_g
modTOM_g = TOMMY[inModule_g, inModule_g];
dimnames(modTOM_g) = list(modProbes_g, modProbes_g) 
dimnames(modTOM_g)
length(rownames(modTOM_g))
rownames(modTOM_g)->green_genes
#red module
inModule_r = is.finite(match(moduleColors, MOD_r))
modProbes_r = II[inModule_r]; 
modProbes_r
modTOM_r = TOMMY[inModule_r, inModule_r];
dimnames(modTOM_r) = list(modProbes_r, modProbes_r) 
dimnames(modTOM_r)
length(rownames(modTOM_r))
rownames(modTOM_r)->red_genes


#1.9  Finding genes with high gene significance and high intramodular connectivity in interesting modules
matchedB<- match(black_genes, colnames(c4))
c4_black<-c4[,matchedB]
matchedG<- match(green_genes, colnames(c4))
c4_green<-c4[,matchedG]
matchedR<- match(red_genes, colnames(c4))
c4_red<-c4[,matchedR]

#1.9.1 Compute Module Membership (i.e., intrramodular connectivity for each module)
datKME_B=signedKME(c4_black, MEs0, outputColumnName="MM.")
datKME_G=signedKME(c4_green, MEs0, outputColumnName="MM.")
datKME_R=signedKME(c4_red, MEs0, outputColumnName="MM.")
abs(datKME_B$MM.black)->MM.black
abs(datKME_R$MM.red)->MM.red
abs(datKME_G$MM.green)->MM.green

#1.9.2 Compute Gene Significance
as.numeric(cor(datTraits$Age,c4_black, use="p"))->GS_B
abs(GS_B)->GeneSignificance_B
as.numeric(cor(datTraits$Age,c4_red, use="p"))->GS_R
abs(GS_R)->GeneSignificance_R
as.numeric(cor(datTraits$Age,c4_green, use="p"))->GS_G
abs(GS_G)->GeneSignificance_G
as.data.frame(cbind(rownames(datKME_B), MM.black, GeneSignificance_B))->MM_GS_B
as.data.frame(cbind(rownames(datKME_G), MM.green, GeneSignificance_G))->MM_GS_G
as.data.frame(cbind(rownames(datKME_R), MM.red, GeneSignificance_R))->MM_GS_R
colnames(MM_GS_B)<-c("gene", "MM", "GS")
colnames(MM_GS_G)<-c("gene", "MM", "GS")
colnames(MM_GS_R)<-c("gene", "MM", "GS")

#1.9.3 Select hubs having high Gene Significance and Module Membership
MM_GS_B[MM_GS_B$MM>0.9 & MM_GS_B$GS>0.7,]->MM_GS_B_filtered
MM_GS_G[MM_GS_G$MM>0.9 & MM_GS_G$GS>0.7,]->MM_GS_G_filtered
MM_GS_R[MM_GS_R$MM>0.9 & MM_GS_R$GS>0.7,]->MM_GS_R_filtered
write.table(MM_GS_B_filtered, file="MM_GS_B_filtered.txt", quote=F, sep='\t', row.names=F)
write.table(MM_GS_G_filtered, file="MM_GS_G_filtered.txt", quote=F, sep='\t', row.names=F)
write.table(MM_GS_R_filtered, file="MM_GS_R_filtered.txt", quote=F, sep='\t', row.names=F)

#1.10 Export modules for Network visualization in Cytoscape
cyt_b = exportNetworkToCytoscape(modTOM_b,edgeFile = paste("PAPER_CytoscapeInput-edges-", paste(MOD_b, collapse="-"), ".txt", sep=""),nodeFile = paste("CytoscapeInput-nodes-", paste(MOD_b, collapse="-"), ".txt", sep=""),weighted = TRUE,threshold = 0.5,nodeNames = modProbes_b,nodeAttr = moduleColors[inModule_b]) 
cyt_g = exportNetworkToCytoscape(modTOM_g,edgeFile = paste("PAPER_CytoscapeInput-edges-", paste(MOD_g, collapse="-"), ".txt", sep=""),nodeFile = paste("CytoscapeInput-nodes-", paste(MOD_g, collapse="-"), ".txt", sep=""),weighted = TRUE,threshold = 0.5,nodeNames = modProbes_g,nodeAttr = moduleColors[inModule_g]) 
cyt_r = exportNetworkToCytoscape(modTOM_r,edgeFile = paste("PAPER_CytoscapeInput-edges-", paste(MOD_r, collapse="-"), ".txt", sep=""),nodeFile = paste("CytoscapeInput-nodes-", paste(MOD_r, collapse="-"), ".txt", sep=""),weighted = TRUE,threshold = 0.5,nodeNames = modProbes_r,nodeAttr = moduleColors[inModule_r]) 


